/*
 * (C) Copyright IBM Corp. 1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.dom.tester;

import com.ibm.dom.util.Digest;
import com.ibm.xml.parser.Parser;
import com.ibm.xml.parser.TXDocument;
import com.ibm.xml.parser.TXElement;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.io.PrintStream;
import java.net.URL;
import java.security.MessageDigest;
import org.w3c.dom.Document;

/**
 * Create digest value table.
 * This program requires IBM XML4J v1.1.15 or later.
 *
 * @version Revision: %M% %I% %W% %Q%
 * @author TAMURA Kent &lt;kent@trl.ibm.co.jp&gt;
 */
public class MakeDigestTable {

    static String algorithm = "MD5";

    static void parse(String systemId) {
        try {
            MessageDigest md = MessageDigest.getInstance(MakeDigestTable.algorithm);

            Parser xml4jp = new Parser(systemId);
            xml4jp.setProcessNamespace(true);
            xml4jp.setWarningNoXMLDecl(false);
            TXDocument xml4jdoc = xml4jp.readStream(xml4jp.getInputStream(systemId, null, systemId));
            if (xml4jdoc == null || xml4jdoc.getDocumentElement() == null) {
                System.err.println(systemId+": xml4j: Null document.");
                return;
            }
            xml4jdoc.setDigestAlgorithm(MakeDigestTable.algorithm);
            byte[] txdigest = ((TXElement)xml4jdoc.getDocumentElement()).getDigest();
            System.out.print(" domhash=\"");
            printByteArraySTDOUT(txdigest);
            System.out.print("\"");

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void printByteArraySTDOUT(byte[] ab) throws IOException {
        for (int i = 0;  i < ab.length;  i ++) {
            System.out.print(tab.charAt((ab[i]>>4)&0x0f));
            System.out.print(tab.charAt(ab[i]&0x0f));
        }
        System.out.flush();
    }
    static final String tab = "0123456789ABCDEF";

    public static String file2URL(File f) {
        String abs = f.getAbsolutePath();
        if (File.separatorChar != '/')
            abs = abs.replace(File.separatorChar, '/');
        if (!abs.endsWith("/") && f.isDirectory())
            abs = abs+"/";
        return (abs.charAt(0) == '/' ? "file://" : "file:///")+abs;
    }

    public static void main(String[] argv) {
        try {
            String[] names = new String[argv.length];
            int nameslen = 0;
            for (int i = 0;  i < argv.length;  i ++) {
                if (argv[i].charAt(0) != '-') {
                    names[nameslen++] = argv[i];
                } else {
                    algorithm = argv[i].substring(1);
                }
            }

            System.out.print("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
            System.out.print("<!DOCTYPE digests SYSTEM \"digests.dtd\">\n");
            System.out.print("<digests algorithm=\"");
            System.out.print(algorithm);
            System.out.print("\">\n");
            for (int i = 0;  i < nameslen;  i ++) {
                System.out.print("  <item src=\"");
                System.out.print(names[i]);
                System.out.print("\"");

                File f = new File(names[i]);

                int len = (int)f.length();
                byte[] buf = new byte[len];
                InputStream is = new FileInputStream(f);
                int read = 0;
                while (read < len) {
                    int n = is.read(buf, read, len-read);
                    if (n < 0) throw new RuntimeException("Unexpected EOF!: "+names[i]);
                    read += n;
                }
                is.close();
                MessageDigest md = MessageDigest.getInstance(algorithm);
                byte[] digest = md.digest(buf);
                System.out.print(" digest=\"");
                printByteArraySTDOUT(digest);
                System.out.print("\"");

                parse(file2URL(f));

                System.out.print("/>\n");
            }
            System.out.print("</digests>\n");
            System.out.flush();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
