/*
 * (C) Copyright IBM Corp. 1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.dom.tester;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.net.MalformedURLException;
import java.security.MessageDigest;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JRootPane;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;
import org.xml.sax.ErrorHandler;
import org.xml.sax.Parser;

/**
 *
 * @version Revision: %M% %I% %W% %Q%
 * @author TAMURA Kent &lt;kent@trl.ibm.co.jp&gt;
 */
public class TestDomhashGUI extends JFrame {

    // ================================================================

    static final String[] parsers = {
        "SAX Parser",
        "XML4J v1 DOM",
        "XML4J v2 DOM",
        "Sun Project X DOM",
        "OpenXML DOM"
    };

    static final int ST_FAILED = 0;
    static final int ST_MATCHED = 1;
    static final int ST_UNMATCHED = 2;

    // ================================================================

    static class SetupDialog extends JDialog implements ActionListener {
        JRadioButton[] buttons;
        JComboBox combo;
        JTextField tableText;
        JButton chooseTable;
        JTextField dirText;
        JButton chooseDir;
        JButton ok;
        JButton cancel;
        boolean complete = true;
        TestDomhash.Config conf = new TestDomhash.Config();

        public SetupDialog(JFrame frame, String title, boolean modal) {
            super(frame, title, modal);
            this.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    SetupDialog.this.setVisible(false);
                    SetupDialog.this.complete = false;
                    SetupDialog.this.applyConfig(SetupDialog.this.conf);
                }
            });

            Container container = this.getContentPane();

            JPanel panelP = new JPanel(new GridLayout(5, 1));
            panelP.setBorder(new TitledBorder("XML Parser"));
            this.buttons = new JRadioButton[5];
            this.buttons[0] = new JRadioButton(TestDomhashGUI.parsers[0], true);
            this.buttons[0].setMnemonic('S');
            this.buttons[1] = new JRadioButton(TestDomhashGUI.parsers[1], false);
            this.buttons[1].setMnemonic('1');
            this.buttons[1].setEnabled(checkClass("com.ibm.xml.parser.Parser"));
            this.buttons[2] = new JRadioButton(TestDomhashGUI.parsers[2], false);
            this.buttons[2].setMnemonic('2');
            this.buttons[2].setEnabled(checkClass("com.ibm.xml.parsers.DOMParser"));
            this.buttons[3] = new JRadioButton(TestDomhashGUI.parsers[3], false);
            this.buttons[3].setMnemonic('P');
            this.buttons[3].setEnabled(checkClass("com.sun.xml.tree.XmlDocument"));
            this.buttons[4] = new JRadioButton(TestDomhashGUI.parsers[4], false);
            this.buttons[4].setMnemonic('O');
            this.buttons[4].setEnabled(checkClass("org.openxml.source.Source"));
            ButtonGroup group = new ButtonGroup();
            for (int i = 0;  i < this.buttons.length;  i ++)
                group.add(this.buttons[i]);
            this.buttons[0].addChangeListener(new ChangeListener() { // SAX Parser
                public void stateChanged(ChangeEvent e) {
                    if (((JRadioButton)e.getSource()).isSelected()) {
                        combo.setEnabled(true);
                    } else {
                        combo.setEnabled(false);
                    }
                }
            });
            this.combo = new JComboBox();
            this.combo.setEditable(true);
            
            checkClass(this.combo, "com.ibm.xml.parser.SAXDriver");
            checkClass(this.combo, "com.ibm.xml.parsers.SAXParser");
            checkClass(this.combo, "com.ibm.xml.parsers.ValidatingSAXParser");
            checkClass(this.combo, "com.jclark.xml.sax.Driver");
            checkClass(this.combo, "com.microstar.xml.SAXDriver");
            checkClass(this.combo, "com.sun.xml.parser.Parser");
            checkClass(this.combo, "com.sun.xml.parser.ValidatingParser");
            if (this.conf.parserClass == null)
                this.conf.parserClass = (String)this.combo.getSelectedItem();
            GridBagLayout gbl = new GridBagLayout();
            GridBagConstraints gbc = new GridBagConstraints();
            gbc.fill = GridBagConstraints.HORIZONTAL;
            JPanel saxpanel = new JPanel(gbl);
            gbc.weightx = 0;
            gbc.gridheight = 1;
            gbc.gridwidth = GridBagConstraints.RELATIVE;
            gbl.setConstraints(this.buttons[0], gbc);
            saxpanel.add(this.buttons[0]);
            gbc.weightx = 1;
            gbc.gridwidth = GridBagConstraints.REMAINDER;
            gbl.setConstraints(this.combo, gbc);
            saxpanel.add(this.combo);
            panelP.add(saxpanel);
            for (int i = 1;  i < this.buttons.length;  i ++)
                panelP.add(this.buttons[i]);


            gbl = new GridBagLayout();
            gbc.fill = GridBagConstraints.HORIZONTAL;
            JPanel panelT = new JPanel(gbl);
            panelT.setBorder(new TitledBorder("Table document URL"));
            this.tableText = new JTextField(32);
            this.tableText.setText("file:///E:/xss4j/data/jc-md5.hash");
            this.chooseTable = new JButton("Choose");
            this.chooseTable.addActionListener(this);
            gbc.weightx = 1;
            gbc.gridheight = 1;
            gbc.gridwidth = GridBagConstraints.RELATIVE;
            gbl.setConstraints(tableText, gbc);
            panelT.add(this.tableText);
            gbc.weightx = 0;
            gbc.gridwidth = GridBagConstraints.REMAINDER;
            gbl.setConstraints(this.chooseTable, gbc);
            panelT.add(this.chooseTable);

            gbl = new GridBagLayout();
            gbc.fill = GridBagConstraints.HORIZONTAL;
            JPanel panelD = new JPanel(gbl);
            panelD.setBorder(new TitledBorder("Base URL for test cases"));
            this.dirText = new JTextField(32);
            this.dirText.setText("file:///E:/xmldocs/xmltest/");
            this.chooseDir = new JButton("Choose");
            this.chooseDir.addActionListener(this);
            gbc.weightx = 1;
            gbc.gridheight = 1;
            gbc.gridwidth = GridBagConstraints.RELATIVE;
            gbl.setConstraints(dirText, gbc);
            panelD.add(this.dirText);
            gbc.weightx = 0;
            gbc.gridwidth = GridBagConstraints.REMAINDER;
            gbl.setConstraints(this.chooseDir, gbc);
            panelD.add(this.chooseDir);

            JPanel panelB = new JPanel();
            this.ok = new JButton("Ok");
            this.ok.addActionListener(this);
            panelB.add(this.ok);
            this.cancel = new JButton("Cancel");
            this.cancel.addActionListener(this);
            panelB.add(this.cancel);

            gbl = new GridBagLayout();
            container.setLayout(gbl);
            //gbc.fill = GridBagConstraints.HORIZONTAL;
            gbc.fill = GridBagConstraints.BOTH;

            gbc.weightx = 1.0;
            gbc.weighty = 2.5;
            gbc.gridwidth = GridBagConstraints.REMAINDER;
            gbc.gridheight = 3;
            gbl.setConstraints(panelP, gbc);
            container.add(panelP);

            gbc.weighty = 1.0;
            gbc.gridheight = 1;
            gbl.setConstraints(panelT, gbc);
            container.add(panelT);
            gbl.setConstraints(panelD, gbc);
            container.add(panelD);
            gbl.setConstraints(panelB, gbc);
            container.add(panelB);

            this.setSize(480, 320);
        }

        static boolean checkClass(String classname) {
            boolean ret = false;
            try {
                Class.forName(classname);
                ret = true;
            } catch (ClassNotFoundException cnfe) {
                System.out.println("checkClass(): "+cnfe);
            }
            return ret;
        }

        static void checkClass(JComboBox combo, String classname) {
            if (checkClass(classname))
                combo.addItem(classname);
        }

        public void actionPerformed(ActionEvent e) {
            Object src = e.getSource();
            if (src == this.ok) {
                if (this.tableText.getText().length() > 0
                    && this.dirText.getText().length() > 0) {
                    this.setVisible(false);
                    this.complete = true;
                    this.flushConfig();
                }
            } else if (src == this.cancel) {
                this.setVisible(false);
                this.complete = false;
                this.applyConfig(this.conf);
            } else if (src == this.chooseTable) {
                JFileChooser chooser = new JFileChooser();
                //chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
                FileFilter ff = new FileFilter() {
                    public boolean accept(File f) {
                        return f.isDirectory() || f.getName().endsWith(".hash");
                    }
                    public String getDescription() {
                        return "Digest value table (*.hash)";
                    }
                };
                chooser.addChoosableFileFilter(ff);
                chooser.setFileFilter(ff);
                chooser.removeChoosableFileFilter(chooser.getAcceptAllFileFilter());
                if (chooser.showDialog(this, "Select") == chooser.APPROVE_OPTION) {
                    File f = chooser.getSelectedFile();
                    this.tableText.setText(MakeDigestTable.file2URL(f));
                }
            } else if (src == this.chooseDir) {
                JFileChooser chooser = new JFileChooser();
                chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                if (chooser.showDialog(this, "Select") == chooser.APPROVE_OPTION) {
                    File f = chooser.getSelectedFile();
                    this.dirText.setText(MakeDigestTable.file2URL(f));
                }
            }
        }

        void applyConfig(TestDomhash.Config conf) {
            for (int i = 0;  i < this.buttons.length;  i ++)
                this.buttons[i].setSelected(conf.type == i);
            if (conf.parserClass != null)
                this.combo.setSelectedItem(conf.parserClass);
            if (conf.table != null)
                this.tableText.setText(conf.table);
            if (conf.base != null)
                this.dirText.setText(conf.base);

            if (this.conf != conf) {
                this.conf.type = conf.type;
                this.conf.parserClass = conf.parserClass;
                this.conf.table = conf.table;
                this.conf.base = conf.base;
            }
        }

        TestDomhash.Config getConfig() {
            return this.conf;
        }

        private void flushConfig() {
            this.conf.type = TestDomhash.T_SAX;
            for (int i = 0;  i < 5;  i ++) {
                if (this.buttons[i].isSelected()) {
                    if (i == 0)
                        conf.type = TestDomhash.T_SAX;
                    else if (i == 1)
                        conf.type = TestDomhash.T_XML4J1;
                    else if (i == 2)
                        conf.type = TestDomhash.T_XML4J2;
                    else if (i == 3)
                        conf.type = TestDomhash.T_PROJX;
                    else if (i == 4)
                        conf.type = TestDomhash.T_OPENX;
                    break;
                }
            }
            this.conf.parserClass = (String)this.combo.getSelectedItem();
            this.conf.table = this.tableText.getText();
            this.conf.base = this.dirText.getText();
        }
    }

    // ================================================================

    static class DigestTableCellRenderer extends DefaultTableCellRenderer {
        public Component getTableCellRendererComponent(JTable table,
                                                       Object value,
                                                       boolean isSelected,
                                                       boolean hasFocus,
                                                       int row,
                                                       int column) {
            Component ret = super.getTableCellRendererComponent(table, value, isSelected,
                                                                hasFocus, row, column);
            ret.setForeground(Color.black);
            if (column == 0)  return ret;
            TableModelImpl impl = (TableModelImpl)table.getModel();
            int state = column == 1 ? impl.digests[row] : impl.domhashs[row];
            if (state == TestDomhashGUI.ST_MATCHED) {
                ret.setForeground(Color.blue);
            } else if (state == TestDomhashGUI.ST_UNMATCHED) {
                ret.setForeground(Color.red);
            }
            return ret;
        }
    }

    // ================================================================

    static class TableModelImpl extends AbstractTableModel
        implements TestDomhash.RawDigestMatchingListener {
        static final String[] ids = {"Resource", "Digest values for raw octets", "DOMHASH", "Status"};
        TestDomhash.DigestTable tab;
        int[] digests;
        int[] domhashs;
        String[] status;
        int counter;

        TableModelImpl(TestDomhash.DigestTable tab) {
            this.tab = tab;
            this.digests = new int[this.tab.items.size()];
            this.domhashs = new int[this.tab.items.size()];
            this.status = new String[this.tab.items.size()];
        }

        void setStatus(int row, String str) {
            this.status[row] = str;
            this.fireTableCellUpdated(row, 3);
        }
        void setStatus(int row, int st) {
            this.domhashs[row] = st;
            this.fireTableCellUpdated(row, 2);
        }

        public void handleUnmatchedDigest(String expected, String result) {
            this.digests[this.counter] = TestDomhashGUI.ST_UNMATCHED;
            this.fireTableCellUpdated(this.counter, 1);
        }

        TestDomhash.Item getRow(int row) {
            return (TestDomhash.Item)this.tab.items.elementAt(row);
        }

        public int getRowCount() {
            return this.tab.items.size();
        }
        public int getColumnCount() {
            return 4;
        }
        public String getColumnName(int column) {
            return ids[column];
        }
        public Object getValueAt(int row, int column) {
            TestDomhash.Item item = (TestDomhash.Item)this.tab.items.elementAt(row);
            switch (column) {
              case 0:
                return item.src;
              case 1:
                return item.digest;
              case 2:
                return item.domhash;
              case 3:
                String st = this.status[row];
                return st == null ? "" : st;
            }
            return null;
        }
  
    }

    // ================================================================

    static final String baseTitle = "TestDomhash";
    SetupDialog dialog;
    JScrollPane scrollPane;
    
    public TestDomhashGUI(String title) {
        super(title);
        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                TestDomhashGUI.this.dispose();
                System.exit(0);
            }
        });
        JMenuBar mb = new JMenuBar();
        JMenuItem menuExit = new JMenuItem("Exit");
        menuExit.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                TestDomhashGUI.this.setVisible(false);
                TestDomhashGUI.this.dispose();
                System.exit(0);
            }
        });
        JMenuItem menuSetup = new JMenuItem("Setup");
        menuSetup.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                TestDomhashGUI.this.setup();
            }
        });
        JMenu m = new JMenu("File");
        m.add(menuSetup);
        m.addSeparator();
        m.add(menuExit);
        mb.add(m);
        this.setJMenuBar(mb);

        this.scrollPane = new JScrollPane();
        this.getContentPane().add(this.scrollPane);

        this.setSize(768, 512);
        dialog = new SetupDialog(this, "Setup", true);
    }
    
    void setup() {
        // Selects a parser and files.
        dialog.show();
        if (dialog.complete) {
            this.process(dialog.getConfig());
        }
    }

    void process(TestDomhash.Config conf) {
        try {
            // Parses a table file.
            String title = TestDomhashGUI.baseTitle;
            if (conf.type == TestDomhash.T_SAX) {
                title = title +": " +conf.parserClass +" / " +conf.table;
            } else {
                title = title +": " +TestDomhashGUI.parsers[conf.type] +" / " +conf.table;
            }
            this.setTitle(title);

            TestDomhash.DigestTable tab = new TestDomhash.DigestTable();
            Parser parser = null;
            ErrorHandler stderr = null;
            if (conf.type == TestDomhash.T_SAX) {
                parser = (Parser)Class.forName(conf.parserClass).newInstance();
                parser.setDocumentHandler(tab);
                parser.setErrorHandler(stderr = new TestDomhash.Stderr());
                parser.parse(conf.table);
            } else {
                tab.fromDOM(TestDomhash.parseDocument(conf.type, conf.table));
            }
                
            // Creates JTable.
            TableModelImpl impl = new TableModelImpl(tab);
            JTable table = new JTable(impl);
            table.setDefaultRenderer(Object.class, new DigestTableCellRenderer());
            table.setFont(new Font("Monospaced", Font.PLAIN, 12));
            int hashlen = ((TestDomhash.Item)tab.items.elementAt(0)).digest.length();
            TableColumn tc = table.getColumnModel().getColumn(0);
            tc.setPreferredWidth(6*tab.maxsrc);
            tc = table.getColumnModel().getColumn(1);
            tc.setPreferredWidth(6*hashlen);
            tc = table.getColumnModel().getColumn(2);
            tc.setPreferredWidth(6*hashlen);
            tc = table.getColumnModel().getColumn(3);
            tc.setPreferredWidth(6*15);
            this.scrollPane.setViewportView(table);
            this.validate();

            // Check documents.
            MessageDigest md = MessageDigest.getInstance(tab.algorithm);
            for (int i = 0;  i < tab.items.size();  i ++) {
                TestDomhash.Item item = (TestDomhash.Item)tab.items.elementAt(i);
                if (item.domhash != null) {
                    impl.counter = i;
                    long start = System.currentTimeMillis();
                    String domhash = TestDomhash.check(item, conf, md, tab.algorithm,
                                                       parser, stderr, impl);
                    long elapsed = System.currentTimeMillis()-start;
                    if (domhash == null) {
                        impl.setStatus(i, "Failed to parse");
                    } else if (!item.domhash.equals(domhash)) {
                        impl.setStatus(i, TestDomhashGUI.ST_UNMATCHED);
                        impl.setStatus(i, "Not matched: "+domhash);
                    } else {
                        impl.setStatus(i, TestDomhashGUI.ST_MATCHED);
                        //impl.setStatus(i, "Ok ("+elapsed+"msec)");
                        impl.setStatus(i, "Ok");
                    }
                }
            }
            System.out.println("Checked all documents.");
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public static void main(String[] argv) {
        TestDomhash.Config conf = null;
        if (argv.length != 0) {
            conf = new TestDomhash.Config(argv);
            if (conf.base == null) {
                System.err.println("java com.ibm.dom.util.TestDomhashGUI");
                System.err.println("java com.ibm.dom.util.TestDomhashGUI <SAX Parser class> <table URL> <documents base URL>");
                System.err.println("java com.ibm.dom.util.TestDomhashGUI [options] <table URL> <documents base URL>");
                TestDomhash.printUsage();
                System.exit(1);
            }
        }

        // Creates a frame.
        TestDomhashGUI frame = new TestDomhashGUI(TestDomhashGUI.baseTitle);
        frame.show();

        if (conf != null)
            frame.dialog.applyConfig(conf);
        frame.setup();
    }
}
