/*
 * (C) Copyright IBM Corp. 1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.dom.util;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

/**
 *
 * @version Revision: %M% %I% %W% %Q%
 * @author TAMURA Kent &lt;kent@trl.ibm.co.jp&gt;
 */
public class Normalizer {
    /**
     * Normalize specified node and its descendants.
     * <ul>
     *  <li>Remove 0-length Text nodes
     *  <li>Replace CDATASection nodes by Text nodes
     *  <li>Join adjoining Text nodes
     *  <li>Remove and expand EntityReference nodes
     * </ul>
     */
    public static Node normalize(Node node) {
        Node last = null;
        Node current = node.getFirstChild();
        while (current != null) {
            int type = current.getNodeType();
            switch (type) {
              case Node.ELEMENT_NODE:
                Normalizer.normalize(current);
                last = current;
                current = current.getNextSibling();
                break;

              case Node.ENTITY_REFERENCE_NODE:
                Node next = current.getNextSibling();
                node.removeChild(current);
                Node child;
                while ((child = current.getFirstChild()) != null)
                    node.insertBefore(child, next);
                current = last == null ? node.getFirstChild() : last.getNextSibling();
                break;

              case Node.TEXT_NODE:
              case Node.CDATA_SECTION_NODE:
                Text text = (Text)current;
                if (text.getLength() == 0) {
                    next = current.getNextSibling();
                    node.removeChild(current);
                    current = next;
                } else if (last != null && last.getNodeType() == Node.TEXT_NODE) {
                    ((Text)last).appendData(text.getData());
                    next = current.getNextSibling();
                    node.removeChild(current);
                    current = next;
                } else if (type == Node.CDATA_SECTION_NODE) {
                    Document factory = node.getOwnerDocument();
                    if (factory == null)
                        factory = (Document)node;
                    last = factory.createTextNode(text.getData());
                    node.replaceChild(last, current);
                    current = last.getNextSibling();
                } else {
                    last = current;
                    current = current.getNextSibling();
                }
                break;

              default:
                last = current;
                current = current.getNextSibling();
            }
        }
        return node;
    }
}
