/*
 * (C) Copyright IBM Corp. 1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.xml.dsig;

import org.w3c.dom.*;
//import com.ibm.xml.parser.*;
import com.ibm.dom.util.DOMDuplicator;

/**
 * @author YAGISHITA Kazuyo &lt;kazuyoy@jp.ibm.com&gt;
 */
class XDSManifest implements XDSNode {

    Document doc;
    Element domTree = null;
    XDSSignatureResource resource;
    XDSAttributes attributes = null;
    XDSInfo originator;
    XDSInfo recipient = null;
    XDSAlgorithm keyAgreementAlgorithm = null;
    XDSAlgorithm signatureAlgorithm;

    XDSManifest(Document doc) {
        setDocument(doc);
    }

    public void setDocument(Document doc) {
        this.doc = doc;
    }

    public void setDOMTree(Element el) {
        domTree = el;
    }
    
    public Element getDOMTree() {
        if ( domTree==null ) { 
            domTree = doc.createElement("dsig:Manifest");
            domTree.appendChild(resource.getDOMTree());
            if ( attributes != null ) domTree.appendChild(attributes.getDOMTree());
            
            Element el = doc.createElement("dsig:OriginatorInfo");
            NodeList nl = originator.getDOMTree().getChildNodes();
            for ( int i=0; i<nl.getLength(); i++ ) {
                el.appendChild(DOMDuplicator.cloneNode(doc, nl.item(i)));
            }
            domTree.appendChild(el);

            if ( recipient != null ) {
                el = doc.createElement("dsig:RecipientInfo");
                nl = recipient.getDOMTree().getChildNodes();
                for ( int i=0; i<nl.getLength(); i++ ) {
                    el.appendChild(DOMDuplicator.cloneNode(doc, nl.item(i)));
                }
                domTree.appendChild(el);
            }
            
            if ( keyAgreementAlgorithm != null ) {
                el = doc.createElement("dsig:KeyAgreementAlgorithm");
                el.appendChild(keyAgreementAlgorithm.getDOMTree());
                domTree.appendChild(el);
            }
            
            el = doc.createElement("dsig:SignatureAlgorithm");
            el.appendChild(signatureAlgorithm.getDOMTree());
            domTree.appendChild(el);
        }
        return (Element)domTree.cloneNode(true);
    }
    
}
