/*
 * (C) Copyright IBM Corp. 1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.xml.dsig;

import org.w3c.dom.*;
import java.net.*;

/**
 * @author YAGISHITA Kazuyo &lt;kazuyoy@jp.ibm.com&gt;
 */
class XDSSignatureResource implements XDSResource {

    Document doc;
    Element domTree = null;
    XDSContentType contentType = null;
    XDSLocator locator;
    XDSDigest digest;

    XDSSignatureResource(Document doc, String locator, XDSAlgorithm algorithm, byte[] value) {
        setDocument(doc);
        this.locator = new XDSLocator(doc, locator);
        digest = new XDSDigest(doc, algorithm, value);
    }
    
    public void setDocument(Document doc) {
        this.doc = doc;
    }
    
    public void setDOMTree(Element el) {
        domTree = el;
    }
    
    public Element getDOMTree() {
        if ( domTree==null ) { 
            domTree = doc.createElement("dsig:Resource");
            if ( contentType != null ) domTree.appendChild(contentType.getDOMTree());
            domTree.appendChild(locator.getDOMTree());
            domTree.appendChild(digest.getDOMTree());
        }
        return (Element)domTree.cloneNode(true);
    }
    
    public void setType(String type) {
        contentType = new XDSContentType(doc, type);
    }
    
    public String getType(){
        return contentType.type;
    }

    public void setEncoding(String encoding) {
        digest.value.encoding = encoding;
    }

    public String getEncoding(){
        return digest.value.encoding;
    }
}
