/*
 * (C) Copyright IBM Corp. 1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.xml.dsig;

import com.ibm.xml.sax.*;
import org.xml.sax.Parser;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXParseException;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.ParserFactory;
import java.io.*;
import java.security.*;
import java.security.cert.Certificate;
import java.util.Vector;

import java.io.OutputStream;
import java.io.FilterOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;

/**
 * @author YAGISHITA Kazuyo &lt;kazuyoy@jp.ibm.com&gt;
 */
public class XDSVerify implements ErrorHandler, NamespaceDocumentHandler {
    
    /**
     * Parse a document and verify
     */
    public static void main(String[] argv) {
        String SaxParserClassName = "com.ibm.xml.parsers.SAXParser"; //default

        if (argv.length < 3) {
            System.err.print("Usage: java XDSVerify ");
            System.err.print("SighedData SecurityContext Policy");
            System.err.println("(SAXParserClass Name)");
            System.exit(1);
        }
        
        try {
            if (argv.length > 3) SaxParserClassName = argv[3];
            Parser parser = ParserFactory.makeParser(SaxParserClassName);
            NamespaceAdapter namespaceAdapter = new NamespaceAdapter(parser);
            XDSVerify verify = new XDSVerify(argv[1], argv[2], SaxParserClassName);
            namespaceAdapter.setDocumentHandler(verify);
            namespaceAdapter.setErrorHandler(verify);
            parser.setDocumentHandler(namespaceAdapter);
            parser.parse(argv[0]);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    public void warning(SAXParseException exception) throws SAXException {
        System.err.println("WARNING: "+exception.getMessage());
    }
    public void error(SAXParseException exception) throws SAXException {
        System.err.println("ERROR: "+exception.getMessage());
    }
    public void fatalError(SAXParseException exception) throws SAXException {
        System.err.println("Fatal Error: "+exception.getMessage());
    }
    
    DigestFilter digestFilter;
    String id;
    String keyStorePath;
    String keyStorePassword;
    //  String allowInvalidDSigTree;
    //  String checkPackageValue;
    //  String atLeastOneValidSignature;

    /**
     * Parse a document and verify
     */
    XDSVerify(String securityContext, String policy, String SaxParserClassName) {
        
        try {
            Parser saxparser = ParserFactory.makeParser(SaxParserClassName);
            
            XDSSecurityContextHandler schandler = new XDSSecurityContextHandler();
            saxparser.setDocumentHandler(schandler);
            saxparser.parse(securityContext);
            
            //      XDSPolicyHandler phandler = new XDSPolicyHandler();
            //      saxparser.setDocumentHandler(phandler);
            //      saxparser.parse(policy);
            
            id = schandler.id;
            keyStorePath = schandler.keyStorePath;
            keyStorePassword = schandler.keyStorePassword;
            //      allowInvalidDSigTree = phandler.allowInvalidDSigTree;
            //      checkPackageValue = phandler.checkPackageValue;
            //      atLeastOneValidSignature) = handler.atLeastOneValidSignature);
            
            digestFilter = new DigestFilter(MessageDigest.getInstance("MD5"));
            
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    boolean xml_package = false;
    String digestAlgorithm = null;
    String signatureAlgorithm = null;
    String encoding = null;
    byte[] package_digest = null;
    byte[] manifest_digest = null;
    String rootTagNameOfPackage = null;
    Vector vector = new Vector();

    public void startElement(String namespace, String prefix, String localPart,
                             String qname, NamespaceAttributeList atts)
        throws SAXException {
        
        digestFilter.startElement(namespace, prefix, localPart, qname, atts);
        
        vector.add(qname);

        if (qname.equals("dsig:Algorithm")) {
            if (vector.contains("dsig:DigestAlgorithm")) 
                digestAlgorithm = atts.getValue("type");
            if (vector.contains("dsig:SignatureAlgorithm"))
                signatureAlgorithm = atts.getValue("type");
        }

        if (qname.equals("dsig:Value")) {
            if ( atts.getLength() > 0 ) 
                this.encoding = atts.getValue("encoding");
        } else if (!xml_package && vector.contains("dsig:Value")) {
            xml_package = true;
            rootTagNameOfPackage = qname;
        }
    }    
    
    public void endElement(String namespace, String prefix, 
                           String localPart, String qname)
        throws SAXException {
        
        digestFilter.endElement(namespace, prefix, localPart, qname);
        
        vector.remove(qname);

        if (qname.equals("dsig:Manifest")) {
            // Obtain the digest value of Manifest element
            manifest_digest = digestFilter.getDigest();
        }

        if (qname.equals(rootTagNameOfPackage) && xml_package) {
            xml_package = false;
            package_digest = digestFilter.getDigest();
        }
    }    
    
    public void characters(char[] ch, int start, int length) throws SAXException {
        
        digestFilter.characters(ch, start, length);
        
        if (vector.contains("dsig:Package") && vector.contains("dsig:Value") && !xml_package ) {
            try {
                MessageDigest md = MessageDigest.getInstance(digestAlgorithm);
                byte[] value;
                if (encoding.equals("base64")) {
                    ByteArrayOutputStream baos = new ByteArrayOutputStream();
                    XDSBase64Decoder dec = new XDSBase64Decoder(baos);
                    dec.write(new String(ch, start, length));
                    value = baos.toByteArray();
                    dec.close();
                } else {
                    value = (new String(ch, start, length)).getBytes();
                }
                package_digest = md.digest(value);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        
        if (vector.contains("dsig:Digest") && vector.contains("dsig:Value")) {
            try {
                byte[] value;
                if (encoding.equals("base64")) {
                    ByteArrayOutputStream baos = new ByteArrayOutputStream();
                    XDSBase64Decoder dec = new XDSBase64Decoder(baos);
                    dec.write(new String(ch, start, length));
                    value = baos.toByteArray();
                    dec.close();
                } else {
                    value = (new String(ch, start, length)).getBytes();
                }
                if (MessageDigest.isEqual(package_digest, value)) {
                    System.out.println("Resource verified.");
                } else {
                    System.err.println("Resource did not verify.");
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        
        if (vector.contains("dsig:Signature") && !vector.contains("dsig:Manifest")) {
            
            /**
             * Verify a signed document
             */
            try {
                // 1. Decode its value of this "dsig:Value" element
                byte[] signature;
                if (encoding.equals("base64")) {
                    ByteArrayOutputStream baos = new ByteArrayOutputStream();
                    XDSBase64Decoder dec = new XDSBase64Decoder(baos);
                    dec.write(new String(ch, start, length));
                    signature = baos.toByteArray();
                    dec.close();
                } else {
                    signature = (new String(ch, start, length)).getBytes();
                }
                
                // 2. Set up verification environment
                KeyStore keyStore = KeyStore.getInstance("jks");
                keyStore.load(new FileInputStream(keyStorePath),keyStorePassword.toCharArray());
                Certificate mycert = keyStore.getCertificate(id);
                if (mycert == null) {
                    System.err.println("Specified key not found in the keyStore.");
                    System.exit(-1);
                }
                Signature sig = Signature.getInstance(signatureAlgorithm);
                
                // 3. Verify the signature
                sig.initVerify(mycert.getPublicKey());
                sig.update(manifest_digest);
                if (sig.verify(signature)) {
                    System.out.println("Signature verified.");
                } else {
                    System.out.println("Signature did not verify.");
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        
    }
    
    public void setDocumentLocator(org.xml.sax.Locator locator) {
        digestFilter.setDocumentLocator(locator);
    }
    public void startDocument() throws SAXException {
        digestFilter.startDocument();
    }
    public void endDocument() throws SAXException {
        digestFilter.endDocument();
    }
    public void ignorableWhitespace(char ch[], int start, int length)
        throws SAXException {
        digestFilter.ignorableWhitespace(ch, start, length);
    }
    public void processingInstruction(String target, String data)
        throws SAXException {
        digestFilter.processingInstruction(target, data);
    }
    
}

class XDSBase64Decoder extends FilterOutputStream {
    
    //  initializition of variable
    
    private static final byte decode_table[] = new byte[256];
    private static final char code_list[] = {
        'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 
        'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 
        'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 
        'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 
        'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 
        'y', 'z', '0', '1', '2', '3', '4', '5', '6', '7', 
        '8', '9', '+', '/'
    };
    static 
    {
        for (int i = 0; i < 255; i++)
            decode_table[i] = -1;
        for (int j = 0; j < code_list.length; j++)
            decode_table[code_list[j]] = (byte)j;
    }
    
    byte decode_buffer[];
    int count;   //  sorce 4bytes data read counter
    
    /**
     * constructor
     */
    XDSBase64Decoder(OutputStream os) {
        super(os);
        decode_buffer = new byte[4];
        count = 0;
    }
    
    /**
     *  decode()
     *    internal Base64 data encode method
     */
    void decode() throws IOException {
        byte byte0 = -1;
        byte byte1 = -1;
        byte byte2 = -1;
        byte byte3 = -1;
        int i = 4;
        
        if (i > 3 && decode_buffer[3] == 61)  //  "61" is "=" charactor
            i = 3;
        if (i > 2 && decode_buffer[2] == 61)
            i = 2;

        switch (i) {
          case 4:
            byte3 = decode_table[decode_buffer[3] & 0xff];
            // fall through
            
          case 3:
            byte2 = decode_table[decode_buffer[2] & 0xff];
            // fall through
            
          case 2:
            byte1 = decode_table[decode_buffer[1] & 0xff];
            byte0 = decode_table[decode_buffer[0] & 0xff];
            break;
		}
        switch (i) {
          case 2:
            out.write((byte)(byte0 << 2 & 0xfc | byte1 >>> 4 & 0x3));
            return;
            
          case 3:
            out.write((byte)(byte0 << 2 & 0xfc | byte1 >>> 4 & 0x3));
            out.write((byte)(byte1 << 4 & 0xf0 | byte2 >>> 2 & 0xf));
            return;
            
          case 4:
            out.write((byte)(byte0 << 2 & 0xfc | byte1 >>> 4 & 0x3));
            out.write((byte)(byte1 << 4 & 0xf0 | byte2 >>> 2 & 0xf));
            out.write((byte)(byte2 << 6 & 0xc0 | byte3 & 0x3f));
            return;
        }
    }
    
    void write(String s) throws IOException {
        byte b[] = new byte[s.length()];
        b = s.getBytes();
        for (int n = 0; n < b.length; n++) {
            if (b[n] == 10 || b[n] == 13) continue;  // ignore EnterCode
            decode_buffer[count] = b[n];
            ++count;
            if ( count >= 4 ) {
                count = 0;
                decode();
            }
        }
    }
    
    public void close() throws IOException {
        if ( count != 0 )
            throw new IOException("Incorrect Base64 data is inputed.");
        try {
            flush();
        } catch (IOException ignored) {
            out.close();
        }
    }
    
}
