/*
 * (C) Copyright IBM Corp. 1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.xml.sax;

import org.xml.sax.DocumentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 *
 * @version Revision: %M% %I% %W% %Q%
 * @author TAMURA Kent &lt;kent@trl.ibm.co.jp&gt;
 * @see org.xml.sax.DocumentHandler
 */
public interface NamespaceDocumentHandler {
    /**
     *
     * @see org.xml.sax.DocumentHandler#setDocumentLocator
     */
    public void setDocumentLocator(Locator locator);

    /**
     *
     * @see org.xml.sax.DocumentHandler#startDocument
     */
    public void startDocument() throws SAXException;

    /**
     *
     * @see org.xml.sax.DocumentHandler#endDocument
     */
    public void endDocument() throws SAXException;

    /**
     *
     * @param namespace Namespace URI of element name; when the element has no namespace, this is null.
     * @param prefix    Prefix of element name; when qname has no preix, this is null.
     * @param localPart Local part of element name; when qname has no prefix, this is the same as qname.
     * @param qname     Literal element name; it is the same as `name' parameter of DocumentHandler#startElement().
     * @param atts
     * @see org.xml.sax.DocumentHandler#startElement
     */
    public void startElement(String namespace, String prefix, String localPart,
                             String qname, NamespaceAttributeList atts)
        throws SAXException;

    /**
     *
     * @see org.xml.sax.DocumentHandler#endElement
     */
    public void endElement(String namespace, String prefix, String localPart, String qname)
        throws SAXException;

    /**
     *
     * @see org.xml.sax.DocumentHandler#characters
     */
    public void characters(char ch[], int start, int length)
        throws SAXException;

    /**
     *
     * @see org.xml.sax.DocumentHandler#ignorableWhitespace
     */
    public void ignorableWhitespace(char ch[], int start, int length)
        throws SAXException;

    /**
     *
     * @see org.xml.sax.DocumentHandler#processingInstruction
     */
    public void processingInstruction(String target, String data)
        throws SAXException;
}

